import { spawn } from "child_process";
import path from "path";
import { chdir } from "process";
import fs from "fs";

function _run_cmd({ dir, cmd }: { dir?: string; cmd: string }) {
  const cwdUrl = process.cwd();
  return new Promise<void>((resolve, reject) => {
    // Change to .cpchain
    dir && chdir(dir);
    // TODO 使用正则表达式优化，解决引号中含空格的情况： "a b"
    const _cmd = cmd.split(" ")[0];
    const _args = cmd.split(" ").filter((_, i) => i > 0);
    let ls;
    if (process.platform === "win32") {
      ls = spawn(process.env.comspec || 'cmd.exe', ["/c", _cmd, ..._args]);
    } else {
      ls = spawn(_cmd, _args);
    }
    ls.stdout.on("data", (data) => {
      process.stdout.write(data)
    });
    ls.stderr.on("data", (data) => {
      console.error(`${data}`, "");
    });
    ls.on("close", (code) => {
      chdir(cwdUrl);
      if (code !== 0) {
        reject(new Error("Failed"));
        return;
      }
      resolve();
    });
  });
}

export async function install_deps(cpchainDir: string, registry?: string) {
  let r = ''
  if (registry) {
    r = `--registry=${registry}`
  }
  await _run_cmd({
    dir: cpchainDir,
    cmd: `npm install ${r}`
  });
  console.log(`Install dependency success`);
}

export async function run_build(cpchainDir: string) {
  await _run_cmd({
    dir: cpchainDir,
    cmd: "yarn build"
  })
  // 将 .dapp 中的静态图片复制到 public 中
  const source = path.join(cpchainDir, '.cpchain', '.dapp')
  const dist = path.join(cpchainDir, '.cpchain', 'public')
  fs.cpSync(source, dist, { recursive: true })
}

export async function run_next(cpchainDir: string) {
  await _run_cmd({
    dir: cpchainDir,
    cmd: "yarn dev"
  })
}

export async function run_watch(dir: string) {
  // fork 两个子进程，一个用于 watch files & build，另一个用于运行 next
  _run_cmd({ dir, cmd: "yarn watch"})
  await run_next(path.join(dir, '.cpchain'))
}

export async function run_git_init(cpchainDir: string) {
  await _run_cmd({
    dir: cpchainDir,
    cmd: "git init"
  })
}
