import path from 'path'
import { fileURLToPath } from 'url';
import pkg from "./package.json" assert { type: "json" }
import ShebangPlugin from 'webpack-shebang-plugin'
import ProgressPlugin from 'progress-webpack-plugin'
import GeneratePackageJsonPlugin from 'generate-package-json-webpack-plugin'
import CopyWebpackPlugin from 'copy-webpack-plugin'

const __dirname = path.dirname(fileURLToPath(new URL(import.meta.url)));

const basePackage = {
  "name": pkg.name,
  "version": pkg.version,
  "description": pkg.description,
  "bin": {
    "cpc-cli": "bin/main.js"
  },
  "main": "bin/main.js",
  "author": pkg.author,
  "license": pkg.license,
  "engines": {
    "node": ">= 18"
  }
}

export default {
  entry: './src/main.ts',
  mode: 'production',
  target: 'node',
  resolve: {
    extensions: ['.js', '.json', '.wasm', '.ts'],
  },
  module: {
    rules: [
      {
        test: /\.tsx?$/,
        use: 'ts-loader',
        exclude: [
          /node_modules/,
          /runtimes/,
          /templates/
        ],
      },
    ],
  },
  plugins: [
    new ShebangPlugin(),
    new ProgressPlugin(true),
    new GeneratePackageJsonPlugin(basePackage),
    new CopyWebpackPlugin({
      patterns: [
          { from: 'templates', to: 'templates', info: (file) => ({ minimized: true }), },
          { from: 'runtimes', to: 'runtimes' },
          { from: 'README.md' }
      ]
  })
  ],
  output: {
    filename: 'bin/main.js',
    path: path.resolve(__dirname, 'dist'),
    clean: true
  },
  watchOptions: {
    ignored: ["tmp", "templates", "dist", "runtimes"]
  }
};
